;; @(#) czech.el -- package for effective writing Czech things in Emacs

;; @(#) $Id: czech.el,v 2.1 1996/07/16 20:52:18 pdm Exp $	
;; @(#) $Keywords: i18n, Czech, keyboard $
;; $KnownCompatibility: 19.28, 19.29, 19.30, 19.31, XEmacs 19.14 $

;; This file is *NOT* part of GNU Emacs nor XEmacs.

;; Copyright (C) 1995, 1996 Milan Zamazal

;; Author:       Milan Zamazal <pdm@fi.muni.cz>
;; Maintainer:   Milan Zamazal <pdm@fi.muni.cz>
;; Remark:       Don't laugh too loudly while reading this file, please.

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation, version 2 of the License.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs and/or this package.  If you did not, write to the
;; Free Software Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;; Commentary:

;; Installation:
;; -------------
;;
;; Put this file to your site-lisp directory.
;;
;; Add to your ~/.emacs the following line:
;;
;;   (load "czech")
;;
;; or if you like standard Czech keyboard, use
;;
;;   (setq cz-keyboard-nonstandard nil)
;;   (load "czech")
;;
;; Add line (*before* load command!)
;;
;;   (add-hook 'cz-load-hook 'cz-default-bindings)
;;
;; to assign some keys.

;; Usage:
;; ------
;; This package is highly configurable and has many functions.  Look
;; at documentation file `emacs-czech.info' which contains complete manual.

;;; History:

;; So long, so very long...

;;; Code:


;;; *** Start ***

(defvar cz-verbose-level 9
  "*Verbosity level of `emacs-czech'.
It should be number between `0' and `9'.
`0' means no messages except errors, `9' means print all messages.
`5' or higher guarantees printing of all warnings.")

(defun cz-message (level &rest message)
  "Prints message if it has enough high level.
`MESSAGE' is given to function `message' just when `LEVEL' is less or equal
than value of the variable `cz-verbose-level'."
  (if (<= level cz-verbose-level)
      (apply 'message message)))

(defvar cz-xemacs (string= (substring (emacs-version) 0 6) "XEmacs")
  "Indicates, whether we are running under XEmacs.")

(if cz-xemacs
    (cz-message
     8
     "Warning: Maybe Czech support does not work properly under XEmacs yet."))

(if (not window-system)
    (cz-message 9 "Czech support: Running under X recommended."))

(defconst cz-version "$Id: czech.el,v 2.1 1996/07/16 20:52:18 pdm Exp $"
  "Latest modification time and version number.")

(defvar cz-load-hook nil
  "Hook run when package has been loaded.")

;;; Internal auxiliary functions

(defun cz-test-emacs-version (major minor)
  "True if Emacs version is at least MAJOR.MINOR."
  (or (> emacs-major-version major)
      (and (= emacs-major-version major) (>= emacs-minor-version minor))))

(defun cz-replace-by-car (list element)
  "Replaces element of the list identifying it by its head."
  (if list
      (if (equal (car element) (car (car list)))
	  (cons element (cdr list))
	(cons (car list) (cz-replace-by-car (cdr list) element)))
    (setq list (list element))))


;;; *** 8-bit characters ***

(standard-display-european t)


;;; *** Fonts ***

;;; Variables defining fonts

(defvar cz-font-default nil
  "*Defines the font used as default.
You must reload emacs-czech so that this change would take effect.")

(defvar cz-font-faces-list nil
  "*Defines the list of fonts used for various faces.
List is of form ( ( FACE-NAME \"FONT-DEFINITION\" ) ... ).
You must reload emacs-czech so that this change would take effect.")

(defvar cz-font-info-list nil
  "*Defines the list of fonts used for info faces.
List is of form ( ( FACE-NAME \"FONT-DEFINITION\" ) ... ).
You must reload emacs-czech so that this change would take effect.")

;; One example, how to set variables defined above in your `.emacs' file
;(setq cz-font-default
;      "-misc-fixed-medium-r-normal-sans-15-*-*-*-c-*-iso8859-2")
;				       ; the very similar font to standard font
;(setq cz-font-faces-list
;      '((modeline "-misc-fixed-medium-r-normal-*-15-*-*-*-c-*-iso8859-2")
;	(highlight "-misc-fixed-medium-r-normal-*-15-*-*-*-c-*-iso8859-2")
;	(region "-misc-fixed-medium-r-normal-*-15-*-*-*-c-*-iso8859-2")
;	(secondary-selection
;	 "-misc-fixed-medium-r-normal-*-15-*-*-*-c-*-iso8859-2")
;	(bold "-misc-fixed-bold-r-normal-*-15-*-*-*-c-*-iso8859-2")
;	(italic "-misc-fixed-medium-r-normal-*-15-*-*-*-c-*-iso8859-2")
;	(bold-italic "-misc-fixed-bold-r-normal-*-15-*-*-*-c-*-iso8859-2")
;	(underline "-misc-fixed-medium-r-normal-*-15-*-*-*-c-*-iso8859-2")))
;(setq cz-font-info-list
;      '((info-xref "-misc-fixed-bold-r-normal--15-*-*-*-c-*-iso8859-2")
;	(info-node "-misc-fixed-bold-r-normal-sans-15-*-*-*-c-*-iso8859-2")
;	(info-menu-5
;	 "-misc-fixed-medium-r-normal-sans-15-*-*-*-c-*-iso8859-2")))

;;; Setting fonts

(if window-system
    (progn
      ;; Set default font (only simple for XEmacs)
      (if cz-font-default
	  (if cz-xemacs
	      ;; XEmacs
	      (set-face-font 'default cz-font-default)
	    ;; GNU Emacs
	    (set-default-font cz-font-default)
	    (setq default-frame-alist
		  (cz-replace-by-car default-frame-alist
				     (cons 'font cz-font-default))))
	(if (not cz-xemacs)
	    (setq default-frame-alist
		  (delete (cons 'font cz-font-default) default-frame-alist))))
      ;; Set fonts of faces
      (mapcar
       (lambda (pair) (set-face-font (car pair) (car (cdr pair))))
       cz-font-faces-list)
      ;; Set fonts for info
      (defun cz-info-hook ()
	"Hook to run info with proper Czech fonts."
	(mapcar
	 (lambda (pair)
	   (set-face-font (car pair) (car (cdr pair))))
	 cz-font-faces-list))
      (add-hook 'Info-mode-hook 'cz-info-hook)))


;;; *** Syntax ***

(if cz-xemacs (require 'case-table))

(if (not cz-xemacs)
    (if (cz-test-emacs-version 19 30)
	(require 'iso02-syn)
      (let ((table (car (standard-case-table))))
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	;; Non Czech but ISO 8859-2 characters
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	(set-case-syntax-pair ? ? table)
	;; I don't know how to set `', but it's probably already set by Emacs
	;; Set new table as standard
	(set-standard-case-table (list table nil nil nil))))
  ;; I don't understand XEmacs case tables (does it work?) - so it's hacked
  (require 'iso02-syn)
  (add-hook 'first-change-hook
	    (function (lambda ()
			(set-case-table
			 (list (car (standard-case-table)) nil nil nil))))))


;;; *** Keyboard ***

;;; Variables involving using of Czech keyboard

(defvar cz-global-mode nil
  "*If set non-`nil', Czech keyboard is global, else buffer local.
Default setting is `nil', i.e. buffer local.
You must reload emacs-czech so that this change would take effect.")

(defvar cz-keyboard-default-on nil
  "*If set non-`nil', Czech keyboard mode is switched on globally.
You must reload emacs-czech so that this change would take effect.")

(defvar cz-minibuffer-nonczech nil
  "*If non-`nil', minibuffer does not inherit cz-keyboard from calling buffer.
If `nil', whenever you enter minibuffer, it is set to cz-keyboard-mode just if
calling buffer is in cz-keyboard-mode.  If variable `cz-keyboard-default-on'
or `cz-global-mode' is non-`nil', the value of `cz-minibuffer-nonczech' doesn't
matter.
You must reload emacs-czech so that this change would take effect.")

(defvar cz-keyboard-nonstandard t
  "*If set non-`nil', nonstandard Czech keyboard is used.
If set just `t', default nonstandard keyboard is used.  If you want to use
your own keyboard, set this variable to any value other than `nil' or `t'.
Standard Czech keyboard is probably defined by some official standard.
As much I know it I defined its bindings here.
Nonstandard Czech keyboard is defined by the following variables:
cz-dead-key-1, cz-dead-key-2, cz-keys, cz-keys-dead-1, and cz-keys-dead-2.
You must reload emacs-czech so that this change would take effect.")

(defvar cz-redefine-alt-digits nil
  "*If true, standard numeric argument `M-<digit>' will be redefined.
Instead of numeric argument function this key sequences will produce digits on
Czech keyboard.
You must reload emacs-czech so that this change would take effect.")

;;; Variables defining Czech keyboard

(setq cz-dead-key-1-default "+")

(setq cz-dead-key-2-default "=")

(setq cz-keys-default
      '(("2" [236])
	("3" [185])
	("4" [232])
	("5" [248])
	("6" [190])
	("7" [253])
	("8" [225])
	("9" [237])
	("0" [233])
	("\[" [250])
	("\]" [249])))

(setq cz-keys-dead-1-default
      '(("c" [232])
	("d" [239])
	("e" [236])
	("n" [242])
	("r" [248])
	("s" [185])
	("t" [187])
	("u" [249])
	("z" [190])
	("C" [200])
	("D" [207])
	("E" [204])
	("N" [210])
	("R" [216])
	("S" [169])
	("T" [171])
	("U" [217])
	("Z" [174])))

(setq cz-keys-dead-2-default
      '(("a" [225])
	("e" [233])
	("i" [237])
	("o" [243])
	("u" [250])
	("y" [253])
	("A" [193])
	("E" [201])
	("I" [205])
	("O" [211])
	("U" [218])
	("Y" [221])))

(defvar cz-dead-key-1 cz-dead-key-1-default
  "*Defines the first dead key for Czech keyboard.
It is bind to \"+\" default.
You must reload emacs-czech so that this change would take effect.")

(defvar cz-dead-key-2 cz-dead-key-2-default
  "*Defines the second dead key for Czech keyboard.
It is bind to \"=\" key default.
You must reload emacs-czech so that this change would take effect.")

(defvar cz-keys cz-keys-default
  "*Defines keys with Czech character.
Binding of these keys will be changed in Czech keyboard mode.
Together with any dead key they return original characters bound to them.
You must reload emacs-czech so that this change would take effect.")

(defvar cz-keys-dead-1 cz-keys-dead-1-default
  "*Key bindings to the first Czech dead key.
You must reload emacs-czech so that this change would take effect.")

(defvar cz-keys-dead-2 cz-keys-dead-2-default
  "*Key bindings to the second Czech dead key.
You must reload emacs-czech so that this change would take effect.")

(defvar cz-redefine-keypad t
  "*If non-`nil', the package is allowed to redefine keypad keys.
If you do not allow to do this, you may have problems to write numbers with
Czech keyboard on the numeric block.
You must reload emacs-czech so that this change would take effect.")

;;; Create proper keyboard

;; Redefine definition variables if some predefined keyboard should be used
(cond
 ((equal cz-keyboard-nonstandard t)
  (setq cz-dead-key-1 cz-dead-key-1-default)
  (setq cz-dead-key-2 cz-dead-key-2-default)
  (setq cz-keys cz-keys-default)
  (setq cz-keys-dead-1 cz-keys-dead-1-default)
  (setq cz-keys-dead-2 cz-keys-dead-2-default))
 ((not cz-keyboard-nonstandard)
  (progn
    (setq cz-dead-key-1 "+")
    (setq cz-dead-key-2 "=")
    (setq cz-keys
	  '(("1" "++") ("2" [236]) ("3" [185]) ("4" [232]) ("5" [248])
	    ("6" [190]) ("7" [253]) ("8" [225]) ("9" [237]) ("0" [233])
	    ("`" "+\"") ("~" "+;") ("!" "+1") ("@" "+2") ("#" "+3")
	    ("$" "+4") ("%" "+5") ("^" "+6") ("&" "+7") ("*" "+8")
	    ("(" "+9") (")" "+0") ("-" "==") ("_" "+%") ("\[" [250])
	    ("{" "+/") ("\]" "+)") ("}" "+(") (";" [249]) (":" "+\"")
	    ("'" [167]) ("\"" "+!") ("<" "+?") (">" "+:") ("/" "+-")
	    ("?" "+_") ("y" "+z") ("Y" "+Z") ("z" "=y") ("Z" "=Y")))
    (setq cz-keys-dead-1
	  '(("c" [232]) ("d" [239]) ("e" [236]) ("n" [242]) ("r" [248])
	    ("s" [185]) ("t" [187]) ("u" [249]) ("y" [190]) ("C" [200])
	    ("D" [207]) ("E" [204]) ("N" [210]) ("R" [216]) ("S" [169])
	    ("T" [171]) ("U" [217]) ("Y" [174])
	    ("z" self-insert-command) ("Z" self-insert-command)))
    (setq cz-keys-dead-2
	  '(("a" [225]) ("e" [233]) ("i" [237]) ("o" [243]) ("u" [250])
	    ("z" [253]) ("A" [193]) ("E" [201]) ("I" [205]) ("O" [211])
	    ("U" [218]) ("Z" [221])
	    ("y" self-insert-command) ("Y" self-insert-command))))))

;; Some auxiliary functions and variables
(setq cz-keypad
      (if cz-redefine-keypad
	  '(([kp-1] "1") ([kp-2] "2") ([kp-3] "3") ([kp-4] "4") ([kp-5] "5")
	    ([kp-6] "6") ([kp-7] "7") ([kp-8] "8") ([kp-9] "9") ([kp-0] "0")
	    ([kp-add] "+") ([kp-subtract] "-") ([kp-multiply] "*")
	    ([kp-divide] "/") ([kp-decimal] "."))
	nil))
(if (not cz-xemacs)
    (defun cz-define-keys (keymap keylist)
      ;; Auxiliary for adding keys to keymap
      (mapcar
       (lambda (pair)
	 (define-key keymap (car pair) (car (cdr pair))))
       keylist))
  ;; Oh, XEmacs is very susceptible to 8-bit characters
  (defun cz-define-keys (keymap keylist)
    (let ((aux))
      (mapcar
       (lambda (pair)
	 (define-key keymap (car pair)
	   (progn
	     (setq aux (car (cdr pair)))
	     (if (arrayp aux)
		 (vector (list (aref aux 0)))
	       aux))))
       keylist))))

;; Create maps for dead keys - the order is important! (Z<->Y)
(setq cz-keymap-dead-1 (make-sparse-keymap))
(cz-define-keys cz-keymap-dead-1
		(list (list cz-dead-key-1 'self-insert-command)))
(mapcar
 (lambda (pair)
   (define-key cz-keymap-dead-1 (car pair) (lookup-key global-map (car pair))))
 cz-keys)
(cz-define-keys cz-keymap-dead-1 cz-keys-dead-1)
;; Add special things for handling keypad - see below for explanation
(mapcar
 (lambda (pair)
   (define-key cz-keymap-dead-1 (car (cdr pair)) 'self-insert-command))
 cz-keypad)
;; The second map
(setq cz-keymap-dead-2 (make-sparse-keymap))
(cz-define-keys cz-keymap-dead-2
		(list (list cz-dead-key-2 'self-insert-command)))
(mapcar
 (lambda (pair)
   (define-key cz-keymap-dead-2 (car pair) (lookup-key global-map (car pair))))
 cz-keys)
(cz-define-keys cz-keymap-dead-2 cz-keys-dead-2)

;; Create basic Czech keymap
(setq cz-keymap (make-sparse-keymap))
(cz-define-keys cz-keymap cz-keys)
;; Add numlock keys (to work everytime as numeric keys)
(mapcar
 (lambda (pair)
   (let ((key (car pair))
	 (code (car (cdr pair))))
     (define-key cz-keymap key (concat cz-dead-key-1 code))
     ;; also for Isearch mode - see below
     (define-key isearch-mode-map key 'isearch-printing-char)))
 cz-keypad)
;; Add dead keys
(cz-define-keys cz-keymap
		(list
		 (list cz-dead-key-1 cz-keymap-dead-1)
		 (list cz-dead-key-2 cz-keymap-dead-2)))
;; Redefine M-<digit> if asked
(if cz-redefine-alt-digits
    (if (not cz-xemacs)
	(progn
	  (define-key cz-keymap "\M-1" (concat cz-dead-key-1 "1"))
	  (define-key cz-keymap "\M-2" (concat cz-dead-key-1 "2"))
	  (define-key cz-keymap "\M-3" (concat cz-dead-key-1 "3"))
	  (define-key cz-keymap "\M-4" (concat cz-dead-key-1 "4"))
	  (define-key cz-keymap "\M-5" (concat cz-dead-key-1 "5"))
	  (define-key cz-keymap "\M-6" (concat cz-dead-key-1 "6"))
	  (define-key cz-keymap "\M-7" (concat cz-dead-key-1 "7"))
	  (define-key cz-keymap "\M-8" (concat cz-dead-key-1 "8"))
	  (define-key cz-keymap "\M-9" (concat cz-dead-key-1 "9"))
	  (define-key cz-keymap "\M-0" (concat cz-dead-key-1 "0")))
      (define-key cz-keymap '(meta ?1) (concat cz-dead-key-1 "1"))
      (define-key cz-keymap '(meta ?2) (concat cz-dead-key-1 "2"))
      (define-key cz-keymap '(meta ?3) (concat cz-dead-key-1 "3"))
      (define-key cz-keymap '(meta ?4) (concat cz-dead-key-1 "4"))
      (define-key cz-keymap '(meta ?5) (concat cz-dead-key-1 "5"))
      (define-key cz-keymap '(meta ?6) (concat cz-dead-key-1 "6"))
      (define-key cz-keymap '(meta ?7) (concat cz-dead-key-1 "7"))
      (define-key cz-keymap '(meta ?8) (concat cz-dead-key-1 "8"))
      (define-key cz-keymap '(meta ?9) (concat cz-dead-key-1 "9"))
      (define-key cz-keymap '(meta ?0) (concat cz-dead-key-1 "0"))))


;;; Define new minor mode: cz-keyboard-mode

;; Define control variable
(defvar cz-keyboard-mode nil
  "Control variable for cz-keyboard-mode.")
(if cz-global-mode
    (kill-local-variable 'cz-keyboard-mode)
  (make-variable-buffer-local 'cz-keyboard-mode)
  (setq-default cz-keyboard-mode cz-keyboard-default-on))

;; Define mode function
(defun cz-keyboard-mode (&optional arg)
  "Mode for using Czech or another user defined keyboard."
  (interactive)
  (if (or (null arg)
	  (not (equal cz-keyboard-mode (> (prefix-numeric-value arg) 0))))
      ;; Mode must be changed
      (progn
	(if (setq cz-keyboard-mode (not cz-keyboard-mode))
	    (run-hooks 'cz-keyboard-mode-hook))
	(force-mode-line-update))))
(defun cz-keyboard-mode-on ()
  "Switches cz-keyboard mode on."
  (interactive)
  (cz-keyboard-mode 1))
(defun cz-keyboard-mode-off ()
  "Switches cz-keyboard mode on."
  (interactive)
  (cz-keyboard-mode -1))

;; Define minor mode keymap - remove old if needed
(setq minor-mode-map-alist
      (cz-replace-by-car minor-mode-map-alist
			 (cons 'cz-keyboard-mode cz-keymap)))

;; Minor mode name at modeline
(if (not (member '(cz-keyboard-mode " CZ") minor-mode-alist))
    ; again prevent multiple things
    (setq minor-mode-alist (cons '(cz-keyboard-mode " CZ") minor-mode-alist)))

;; Function for defining some key bindings
(defun cz-default-bindings ()
  "Assigns some keys to various emacs-czech package functions.
To enable automatic assigning, add to following to your startup file:
  (add-hook 'cz-load-hook 'cz-default-bindings)"
  (if (not cz-xemacs)
      (if (string= (getenv "TERM") "linux")
	  (progn
	    (global-set-key "\e[P" 'cz-keyboard-mode)
	    (global-set-key "\e\e[P" 'cz-correct-word))
	(global-set-key [pause] 'cz-keyboard-mode)
	(global-set-key [escape pause] 'cz-correct-word)
	(global-set-key [M-pause] 'cz-correct-word))
    (if (string= (getenv "TERM") "linux")
	(progn
	  (global-set-key "\e[P" 'cz-keyboard-mode)
	  (global-set-key "\e\e[P" 'cz-correct-word))
      (global-set-key [pause] 'cz-keyboard-mode)
      (global-set-key '(meta pause) 'cz-correct-word))))

;;; Minibuffer handling - let's inherit mode from regular buffer

(defun cz-minibuffer-hook ()
  "Set proper kind of keyboard when entering minibuffer."
  (let (czech-keyboard)
    (save-excursion
      (set-buffer (other-buffer nil t)) ; hopefully sets "calling" buffer
      (setq czech-keyboard cz-keyboard-mode))
    (if czech-keyboard
	(cz-keyboard-mode 1)
      (cz-keyboard-mode -1))))
(if (and (not cz-minibuffer-nonczech)
	 (not cz-global-mode)
	 (not cz-keyboard-default-on))
    (add-hook 'minibuffer-setup-hook 'cz-minibuffer-hook)
  (remove-hook 'minibuffer-setup-hook 'cz-minibuffer-hook))

;;; Isearch handling - the most difficult thing
;; Sorry, because of XEmacs (grr, hacked thing!) I must *redefine directly*
;; `isearch-printing-char' function

;; Firstly redefine sensitive keys
(mapcar
 (lambda (key) (define-key isearch-mode-map key 'isearch-printing-char))
 (append (list cz-dead-key-1 cz-dead-key-2)
	 (mapcar (lambda (pair) (car pair)) cz-keys)
	 (mapcar (lambda (pair) (car pair)) cz-keys-dead-1)
	 (mapcar (lambda (pair) (car pair)) cz-keys-dead-2)))

;; Auxiliary function for handling keypad keys
(defun cz-isearch-keypad (char)
  (let ((list cz-keypad))
    (while (not (equal (aref (car (car list)) 0) char))
      (setq list (cdr list)))
    (aref (car (cdr (car list))) 0)))
;; Define new function handling Czech keys
(defun isearch-printing-char ()
  "Special handling potentially Czech chars instead of
`isearch-printing-char'."
  (interactive)
  (if cz-keyboard-mode
      (let ((key-string)
	    (key)				; found key
	    (dead))				; flag - is it dead key?
	;; Keypad key; following is very stupid but I couldn't find how to do
	;; it more simply - !!key-translation-map!!
	;; Other problem: XEmacs returns just number even for keypad keys in
	;; `last-command-char' - so keypad doesn't work like keypad in XEmacs
	(if (symbolp last-command-char)
	    (setq key (cz-isearch-keypad last-command-char))
	  (progn
	    (setq key-string (char-to-string last-command-char))
	    (cond
	     ;; The key after some dead key
	     ((setq key (lookup-key cz-isearch-dead-map key-string))
	      (setq key (aref (if (symbolp key) key-string key) 0)))
	     ;; First dead key
	     ((equal key-string cz-dead-key-1)
	     (setq cz-isearch-dead-map cz-keymap-dead-1)
	     (setq dead t))
	     ;; Second dead key
	     ((equal key-string cz-dead-key-2)
	     (setq cz-isearch-dead-map cz-keymap-dead-2)
	     (setq dead t))
	     ;; Another key
	     (t
	     (if (setq key (lookup-key cz-keymap key-string))
		 (setq key (aref key 0))
	       (setq key last-command-char))))))
	(if dead
	    (isearch-update)		; keep message at minibuffer line
	  (progn
	    (setq cz-isearch-dead-map (make-sparse-keymap))
					; no current dead key
	    (if (and cz-xemacs (listp key))
		(setq key (car key)))
	    (isearch-process-search-char key)))) ; continue isearch process
    ;; not cz-keyboard-mode - simply call the original function
    (progn
      (if (symbolp last-command-char)
	  (setq last-command-char (cz-isearch-keypad last-command-char)))
      (isearch-process-search-char last-command-event))))

(defun cz-isearch-mode-hook ()
  "Hook for using Czech keyboard in isearch."
  (setq cz-isearch-dead-map (make-sparse-keymap))) ; no current dead key
(add-hook 'isearch-mode-hook 'cz-isearch-mode-hook)


;;; *** Utilities ***

(setq cz-correct-table-on (make-string 256 0))
(let ((i 0))
  (while (< i 256)
    (aset cz-correct-table-on i i)
    (setq i (1+ i))))
(let ((list cz-keys))
  (while list
    (aset cz-correct-table-on
	  (string-to-char (car (car list))) (elt (car (cdr (car list))) 0))
    (setq list (cdr list))))
(setq cz-correct-table-off (make-string 256 0))
(let ((i 0))
  (while (< i 256)
    (aset cz-correct-table-off i i)
    (setq i (1+ i))))
(let ((list cz-keys))
  (while list
    (aset cz-correct-table-off
	  (elt (car (cdr (car list))) 0) (string-to-char (car (car list))))
    (setq list (cdr list))))

(defun cz-correct-word-on (&optional count)
  "Translates last word(s) typed on non-Czech keyboard.
It translates last `COUNT' words typed on non-Czech keyboard as they would be
typed on Czech keyboard.  `COUNT' must be positive.
It does not translate all characters, but most of those defined in the list
`cz-keys'."
  (interactive "*p")
  (if (not count)
      (setq count 1)
    (if (<= count 0) (error "Positive argument expected.")))
  (save-excursion
    (let ((end (point)))
      (forward-word (- count))		; faster than `backward-word'
      (translate-region (point) end cz-correct-table-on))))

(defun cz-correct-word-off (&optional count)
  "Translates last word(s) typed on Czech keyboard.
It translates last `COUNT' words typed on Czech keyboard as they would be
typed on non-Czech keyboard.  `COUNT' must be positive.
It does not translate all characters, but most of those defined in the list
`cz-keys'."
  (interactive "*p")
  (if (not count)
      (setq count 1)
    (if (<= count 0) (error "Positive argument expected.")))
  (save-excursion
    (let ((end (point)))
      (forward-word (- count))		; faster than `backward-word'
      (translate-region (point) end cz-correct-table-off))))

(defun cz-correct-word (&optional count)
  "Translates last word(s) typed on other keyboard.
It translates last `COUNT' words typed on other keyboard as they would be
typed on current keyboard.  `COUNT' must be positive.
It does not translate all characters, but most of those defined in the list
`cz-keys'."
  (interactive "*p")
  (if cz-keyboard-mode
      (cz-correct-word-on count)
    (cz-correct-word-off count)))


;;; *** Goodies ***

;;; Testing for Czech text

(defvar cz-is-empty-czech nil
  "Whether the empty buffer is Czech text.")

(defvar cz-is-buffer-empty 80
  "How many bytes can have buffer to be considered empty.")

(defvar cz-is-test-size 1000
  "Maximal count of tested bytes.")

(defvar cz-is-skip-start 0
  "How many bytes to ignore at the beginning of the buffer.")

(defvar cz-is-czech-min 0.01
  "The minimal ratio of \"Czech\" characters in the tested part of the text.
The text is considered to be Czech only if this ratio is reached at least.
\"Czech\" characters are those from above half of the character set contained
in the list `cz-is-czech-characters'.")

(defvar cz-is-nonczech-max 0.01
  "The maximal ratio of \"non-Czech\" characters in the tested part of the text.
The text is considered to be Czech only if this ratio is not exceeded.
\"nonCzech\" characters are those from above half of the character set not
contained in the list `cz-is-czech-characters'.")

(defvar cz-is-czech-characters
  '(? ? ? ? ? ? ? ? ? ? ? ? ? ? ?
    ? ? ? ? ? ? ? ? ? ? ? ? ? ? ?)
  "List of characters to be considered \"Czech\".")

(defun cz-is-cz-text (&optional return-ratio test-all)
  "Tries to recognize, whether current buffer contains ISO-8859-2 text.
If the buffer is narrowed, only the narrowed part is consedered.
If optional argument `RETURN-RATIO' is non-`nil', ratio (floating point number)
is returned instead of boolean value.  Ratio belongs to interval [-1;1] and
equals to zero just when number of Czech and non-Czech characters from the
above half of character set is the same.
If `TEST-ALL' is non-`nil', test region without considering variables
`cz-is-buffer-empty' and `cz-is-skip-start' (but `cz-is-test-size' is always
considered).
The recognition is absolutely magical and nothing should hard depend on it.
See also variable `cz-is-czech-characters'."
  (let ((size (- (point-max) (point-min))))
    (if (or (eq size 0)
	    (and (null test-all)
		 (<= size cz-is-buffer-empty)))
	(if return-ratio 0 cz-is-empty-czech)
      (save-excursion
	(let ((czech-chars 0)		; score of high Czech chars found
	      (nonczech-chars 0)	; score of high non-Czech chars found
	      (char-count 0)		; number of characters tested
	      (char)
	      (ratio)
	      (ratio-nonczech))
	  (goto-char (if test-all
			 (point-min)
		       (min (+ (point-min) cz-is-skip-start) (point-max))))
	  (while (and (<= (point) (point-max))
		      (<= char-count cz-is-test-size))
	    (setq char (following-char))
	    (if (>= char 128)
		(if (member char cz-is-czech-characters)
		    (setq czech-chars (1+ czech-chars))
		  (setq nonczech-chars (1+ nonczech-chars))))
	    (setq char-count (1+ char-count))
	    (if (< (point) (point-max))
		(forward-char)))
	  (setq ratio (/ (float czech-chars) size))
	  (setq ratio-nonczech (/ (float nonczech-chars) size))
	  (if return-ratio
	      (- ratio ratio-nonczech)	; it could be OK in small regions
	    (and (>= ratio cz-is-czech-min)
		 (<= ratio-nonczech cz-is-nonczech-max))))))))


;;; *** Misc ***

;; Make life easier for users
(autoload 'cz-convert-to-csech "czech-conv" nil t)
(autoload 'cz-convert-to-iso "czech-conv" nil t)
(autoload 'cz-convert-from-to "czech-conv" nil t)


;;; *** Announce ***

(provide 'czech)


;;; *** Run user hooks ***

(run-hooks 'cz-load-hook)


;;; czech.el ends here

