;;;; Installation procedure for `emacs-czech' package.
;;;; Copyright (C) 1996 Milan Zamazal
;;;; $Id: install-czech.el,v 2.1 1996/07/16 20:57:26 pdm Exp $

(setq cz-verbose-level 0)


(defvar lisp-files
  '("czech.el" "czech-conv.el")
  "List of elisp files for removing.")

(defvar info-files
  '("emacs-czech.info")
  "List of info files for removing.")

(defvar del-lisp-files
  '("czech.el" "czech.elc" "czech-conv.el" "czech-conv.elc")
  "List of elisp files for removing.")

(defvar del-info-files
  '("czech.info" "czech.info.gz" "emacs-czech.info" "emacs-czech.info.gz")
  "List of info files for removing.")


(defun install ()
  "Install `emacs-czech' package."
  (message "Installing emacs-czech...")
  (let ((lisp-dir (find-lisp-dir))
	(info-dir (find-info-dir))
	(files)
	(f)
	(aux))
    ;; Test for `iso02-syn.el'
    (if (and (or (string= (substring (emacs-version) 0 6) "XEmacs")
		 (and (>= emacs-major-version 19)
		      (>= emacs-minor-version 30)))
	     (not (or (find-lisp-file "iso02-syn.elc")
		      (find-lisp-file "iso02-syn.el"))))
	(progn
	  (message "Missing file iso02-syn.el[c].")
	  (copy-file "iso02-syn.el" (concat lisp-dir "iso02-syn.el"))
	  (message
	   "(If warnings about free variables will appear, ignore them.)")
	  (byte-compile-file (concat lisp-dir "iso02-syn.el"))
	  (message "File iso02-syn.el installed.")))
    ;; Install `czech.el' and `czech-conv.el' files
    (setq files del-lisp-files)
    (while files
      (setq f (car files))
      (while (setq aux (find-lisp-file f))
	(delete-file aux)
	(message "File %s removed." aux))
      (setq files (cdr files)))
    (if (null lisp-dir)
	(error "No Emacs LISP directory found!")
      (setq files lisp-files)
      (while files
	(setq f (car files))
	(copy-file f (concat lisp-dir f))
	(message
	 "(If warnings about free variables will appear, ignore them.)")
	(byte-compile-file (concat lisp-dir f))
	(message "File %s installed." f)
	(setq files (cdr files))))
    ;; Install info files
    (setq files del-info-files)
    (while files
      (setq f (car files))
      (while (setq aux (find-info-file f))
	(delete-file aux)
	(message "File %s removed." aux))
      (setq files (cdr files)))
    (if (null info-dir)
	(error "No info directory found!")
      (setq files info-files)
      (while files
	(setq f (car files))
	(copy-file f (concat info-dir f))
	(condition-case nil
	    (call-process "gzip" nil nil nil "-9" (concat info-dir f))
	  (error nil))
	(message "File %s installed." f)
	(setq files (cdr files))))
    ;; Update `dir'
    (find-file (concat info-dir "dir"))
    (if (not (re-search-forward "(emacs-czech)." nil t))
	(progn
	  (if (re-search-forward "(czech)." nil t)
	      (replace-match "(emacs-czech)." t t)
	    (progn
	      (goto-char (point-max))
	      (insert "
* Emacs Czech: (emacs-czech).		Czech support for GNU Emacs.
")))
	  (save-buffer)
	  (message "File %s updated." (concat info-dir "dir"))))
    ;; OK, all done
    (message "Installation completed.")
    (message "(Lisp files installed to directory %s.)" lisp-dir)
    (message "(Info files installed to directory %s.)" info-dir)))


(defun uninstall ()
  "Remove all files from `emacs-czech' package."
  (message "Why uninstall?")
  (message "Sorry, this capability is not implemented yet."))


(defun find-lisp-file (file)
  "Returns full path to FILE if found in lisp directories."
  (find-it file (cons "/usr/local/site-lisp" load-path)))
	
(defun find-info-file (file)
  "Returns full path to FILE if found in info directories."
  (find-it file (if (boundp 'Info-directory-list)
		    Info-directory-list
		  Info-default-directory-list)))

(defun find-it (file path-list)
  "Returns full path to FILE if found in PATH-LIST directories."
  (let ((f))
    (while path-list
      (if (file-exists-p (concat (ok-dir (car path-list)) file))
	  (progn
	    (setq f (concat (ok-dir (car path-list)) file))
	    (setq path-list nil))
	(setq path-list (cdr path-list))))
    f))

(defun find-lisp-dir ()
  "Find \"the best\" site-lisp directory."
  (if (null load-path)
      nil
    (let* ((path load-path)
	   (best-dir (car path))
	   (best-score (lisp-dir-score best-dir))
	   (dir)
	   (score))
      (setq path (cdr path))
      (while path
	(setq dir (car path))
	(setq score (lisp-dir-score dir))
	(if (> score best-score)
	    (progn
	      (setq best-dir dir)
	      (setq best-score score)))
	(setq path (cdr path)))
      (ok-dir best-dir))))

(defun lisp-dir-score (dir)
  "Goodnes of lisp dir.
site-lisp ->      +4
local ->          +2
emacs ->          -1
version-number -> -8
not exists ->  -1000"
  (let ((score 0))
    (if (and (>= (length dir) 11)
	     (or (string= (substring dir -10 nil) "/site-lisp")
		 (string= (substring dir -11 nil) "/site-lisp/")))
	(setq score (+ score 4)))
    (if (string-match "/local/" dir)
	(setq score (+ score 2)))
    (if (string-match "emacs/" dir)
	(setq score (- score 1)))
    (if (string-match
	 (concat "/" emacs-major-version "." emacs-minor-version "/") dir)
	(setq score (- score 8)))
    (if (not (file-accessible-directory-p dir))
	(setq score (- score 1000)))
    score))

(defun find-info-dir ()
  "Find \"the best\" local info directory,"
  (let* ((path (if (boundp 'Info-directory-list)
		   Info-directory-list
		 Info-default-directory-list))
	 (best-dir (car path))
	 (best-score (info-dir-score best-dir))
	 (dir)
	 (score))
    (if (null path) nil)
    (setq path (cdr path))
    (while path
      (setq dir (car path))
      (setq score (info-dir-score dir))
      (if (> score best-score)
	  (progn
	    (setq best-dir dir)
	    (setq best-score score)))
      (setq path (cdr path)))
    (ok-dir best-dir)))

(defun info-dir-score (dir)
  "Goodnes of info dir.
info ->           +4
local ->          +2
emacs ->          -1
version-number -> -8
not exists ->  -1000"
  (let ((score 0))
    (if (and (>= (length dir) 6)
	     (or (string= (substring dir -5 nil) "/info")
		 (string= (substring dir -6 nil) "/info/")))
	(setq score (+ score 4)))
    (if (string-match "/local/" dir)
	(setq score (+ score 2)))
    (if (string-match "emacs/" dir)
	(setq score (- score 1)))
    (if (string-match
	 (concat "/" emacs-major-version "." emacs-minor-version "/") dir)
	(setq score (- score 8)))
    (if (not (file-accessible-directory-p dir))
	(setq score (- score 1000)))
    score))

(defun ok-dir (dir)
  "Appends `/' to DIR if needed."
  (if (string= (substring dir -1 nil) "/")
      dir
    (concat dir "/")))

